 ** Clears the output and log windows and recalls the program;
DM 'OUT;CLEAR;LOG;CLEAR;PGM;RECALL';

/* ---------------------------------------------------------
 program:       PBC0.SAS
 input:         pbc.dat  DATA A
 output:
 does:          Cox regression
------------------------------------------------------------ */;

*Creates a macro var for location of project work;
%LET PATH=A:       ;

OPTIONS PS=68 LS=120 PAGENO=1 NODATE NOCENTER ERRORS=3;

* automatically prints the date and time on the footnote of output;
FOOTNOTE  "PBC0.SAS      &SYSTIME    &SYSDATE";
TITLE 'Cox Regression';

* Designates file names to store the program, the output and the log;
FILENAME PROGRAM "&PATH\MYPROG\PBC0.SAS";
FILENAME LOG     "&PATH\MYPROG\PBC0.LOG";
FILENAME OUTPUT  "&PATH\MYPROG\PBC0.OUT";
* Saves the program to above designated file;
DM 'PGM; FILE PROGRAM REPLACE';

* Designates lib name where dataset is stored;
LIBNAME DATA "&PATH\DISTRIB\";

**************************************************;
DATA A;
  SET DATA.PBC;
  * create dummy vars for edema levels;
  edema0_5=(edema=0.5);
  edema1=(edema=1);
  run;
**************************************************;

title2 "Measure the treatment effect(Z1)";
proc phreg data=A;
  model time*delta(0)=trt;
run;

* The trial supports not using
* the drug in this disease;

***********************************************;
* The data on the 312 PBC randomized patients can be ;
* used to build a statistical model for the influence;
* of covariates on disease outcome;

title2 "A two-group comparison of the sexes(Z3)";
* Score test is logrank test;
* compare it with likelihood ratio test;

proc phreg data=A;
  model time*delta(0)=sex;
run;

*************************************************;
title2 'Look at the presence of edema(Z7)';
* Score test of the grouped linear version is the
* trend test. Compare with the likelihood ratio test;

proc phreg data=A;
  model time*delta(0)=edema;
run;

***********************************************;
* Now factor edema, and perform the score and LR tests
* of heterogeneity. Is there strong evidence for  ;
* departure from linear trend?  ;

title2 'Edema factored';
proc phreg data=A;
  model time*delta(0)=edema0_5 edema1;
run;

title2 'Likelihood Ratio test of edema linear vs. factored';
* cut and past LR, compute p-value;
data LRtest;
   LR = (1223.840 - 1222.391);
   pvalue=1-probchi(LR,1);
run;
proc print data=LRtest; run;

***********************************************;
title2 'Look at the continuous covariate age(Z2)';
* The score test has no counterpart in K group methods;

proc phreg data=A;
  model time*delta(0)=age;
run;

* What is the interpretation of the risk ratio?   ;

***********************************************;
title2 "Serum bilirubin level(Z8)";
* Is it related to the risk of death due to PBC?
* Start without any adjustments and then look at
* the effect of adding adjustment variables.;

proc phreg data=A;
  model time*delta(0)=bili;
run;

* What is the interpretation of this hazard ratio?

***********************************************;
title2 "Serum bilirubin level(Z8), adjusted";
* Now adjust for sex, age, edema and trt    ;
* and see if the relationship persists      ;

title3 "Ajusted model without bilirubin";
proc phreg data=A;
  model time*delta(0)= sex age edema trt;
run;

title3 "Ajusted model with bilirubin";
proc phreg data=A;
  model time*delta(0)= sex age edema trt bili;
run;

title2 'Likelihood Ratio test of bilirubin';
* cut and past LR, compute p-value;
data LRtest;
   LR = (1205.401 - 1146.865 );
   pvalue=1-probchi(LR,1);
run;
proc print data=LRtest; run;

* Questions:
* 1) How do you intepret the hazard ratio for bilirubin?
* 2) Why do you think bilirubin is related to the
        risk of death due to PBC?
* 3) What else should you do to perform
*       a thorough analysis of these data?  ;

***********************************************;
title2 'Baseline Estimates';
data null;
  input sex age edema trt bili;
  cards;
  0 0 0 0 0
        ;
run;

proc phreg data=A;
  model time*delta(0)= sex age edema trt bili;
  baseline out=B  covariates=null survival=s
                logsurv=ls loglogs=lls;
run;
* note if covariates= is ommitted, the baseline estimates
* are computed only at the mean value of each covariate  ;

**********************************************;
** Set graphic control information ;
** delete work graphics catalog from previous runs;
proc greplay nofs  igout=mygraphs;
   delete _all_;
quit;
proc greplay nofs igout=gseg2;
   delete _all_;
quit;
  goptions reset=(symbol, pattern, footnote) graphrc
        hpos=0 vpos=0  htext=3 pct
        ctext= target= gaccess= gsfmode= noborder
        cback=white ftext=swissb;
  goptions norotate device=win ;
  *goptions rotate  device=hpljs2;
  symbol1 c=DEFAULT v=NONE   i=steplj l=1;
  symbol2 c=BLUE    v=NONE   i=steplj l=3;
legend1 position=( bottom  left  inside )
        value=(t=1 "at 0 covars"  t=2 "at avg. covars")
        label=('Type') mode=protect ;
legend2 position=(top  left  inside )
        value=(t=1 "at 0 covars"  t=2 "at avg. covars")
        label=('Type') mode=protect;
axis1 label=none;
************************************************;

title1 h=5 pct "Cox regression for PBC data";
title2 'Baseline Cumulative hazard';
data C;
  set B;
 ls=-ls;
   *baseline at 0 and average covariates;
   *in same dataset.  Distinguish;
 if age = 0 then type='at 0';
            else type='avg';
run;
proc gplot data=C gout=mygraphs;
  plot ls*time=type / legend=legend2 vaxis=axis1;
run;

title1 h=5 pct 'Cox regression for  PBC data';
title2 'Baseline Survival Function';
proc gplot data=C gout=mygraphs;
  plot s*time=type / legend = legend1 vaxis=axis1;
run;
quit;

***************************************;
** Combine graphs on single pages       ;

GOPTIONS DEVICE=win ROTATE=LANDSCAPE
        HSIZE=10.67in VSIZE=8in;
*GOPTIONS DEVICE=LXLP4037 HSIZE=11 VSIZE=8.5 ROTATE=LANDSCAPE;
PROC GREPLAY NOFS GOUT=GSEG2;
  IGOUT=MYGRAPHS;
  TC = DATA.TEMPLATE;
  TEMPLATE=NEW2H;
  TREPLAY 1:GPLOT1  2:GPLOT;
RUN;
QUIT;

** Automatically saves contents of the output and log windows;
DM 'LOG; FILE LOG    REPLACE';
DM 'OUT; FILE OUTPUT REPLACE';
